*-- File: DateBtns.cc

*----------------------------------------------------------------------
*-- Copyright (c) 1997, 1998 BoKen Enterprises
*
*-- Programmer: 
*
*	Robert W. Newman	rnewman@mindspring.com
*
*-- History: 
*
*	12/06/1997 -- Created version 1.0
* 	12/16/1997 -- Version 1.1 
*		Added code segment for insuring that the FmtDate.dbf
* 		exists. (using Romain Strieff's cretable.prg)
*	12/18/1997 -- Version 1.2
*		Removed Close Database statement and replaced with code
*		that would allow the buttons to be placed on a form
*		with an open DBF file.
*
*		Added code to allow for the possibility that the user
*		would have a table open when the OnOpen event fired
*		for the buttons. Accordingly, the code checks the current
*		Alias() and saves it to a memory variable, and then
*		restores that environment after the button properties
*		have been set.
*
*		The buttons will now work on a form that has an open
*		DBF file, and will Select that Alias after the form
*		has been refreshed with the new date format.
*	01/18/98 -- Version 1.3
*		Added Append Blank if RecCount() is zero. There
*		must be one record in the DBF to accept data.
*	
*-- Description:
*
*	This .CC file is designed to give the designer access to
*	three buttons to modify the date format in an application.
*	MdyBtn toggles between MDY and DMY.
*	CentBtn toggles between YY and YYYY format.
*	MarkBtn togles between / and . as the separator character.
*	The .CC uses FmtDate.DBF, which should contain fields:
*	MDYDMY 	C(3)	&& value MDY or DMY
*	CENTURY	C(3)	&& value ON or OFF
*	MARK	C(1)	&& value / or .
*	(This dbf will be created if it does not exist.)
*
*-- Usage: 
*
*	Add this .CC as a custom control to the control palette.
*	It provides 3 buttons for modifying the date format.
*
*	The file FmtDate.DBF should be in the app directory. It
*	will be created if it does not already exist.
*
*	There need not be a code segment in your opening .PRG file
*	to read the contents of the FmtDate.dbf and SET the 
*	parameters to the values chosen by the user during the
*	previous session. This will be done by Init() procedure.
*
*	The .CC file will check the state of the SET values for 
*	MDY, Century, and Mark and adjust the text on the 
*	buttons accordingly.
*
*
*-- Properties:   
*    
*    
*
*----------------------------------------------------------------------


*-- Button to toggle between mm/dd/yy and dd/mm/yy format

CLASS MDYDMYBUTTON(FormObj,Name) OF PUSHBUTTON(FormObj,Name) Custom
   this.Top = 16
   this.Text = "mdy"
   this.SpeedTip = "Toggle the date format between M/D/Y and D/M/Y."
   this.OnClick = CLASS::MDYDMYBUTTON_ONCLICK
   this.Left = 20
   this.Group = .T.
   this.Height = 1.53
   this.Width = 13
   this.UpBitmap = "RESOURCE #625"	&&#1130"
   this.OnOpen = CLASS::OnOpen
   
   Procedure OnOpen
      Class::Init()			&& Create FmtDate.dbf if it doesn't exist
      Private cAlias
      cAlias = Alias()			&& Assume user has a table open
      Use FmtDate In Select()
      Select FmtDate
      If RecCount() = 0			&& Need one record in DBF
         Append Blank
      Endif
      If Empty(MDYDMY)			&& Read from the MyApp.INI file
         If Set("Date") = "MDY"
            this.Text = "dmy"
         Else
            this.Text = "mdy"
         Endif
      Else				&& Read from the FmtDate.dbf file
         If FmtDate->MDYDMY = "MDY"
            Set Date To MDY
            this.text = "dmy"
            this.Speedtip = "Toggle the date format to D/M/Y"
         Else
            Set Date To DMY
            this.text = "mdy"
            this.Speedtip = "Toggle the date format to M/D/Y"
         Endif
      Endif
      Use in FmtDate
      If .Not. Empty (cAlias)		&& User did have a table open
         Select &cAlias
      Else				&& There were no tables open
         Select Select()
      Endif

   Procedure MDYDMYBUTTON_OnClick
      cAlias = Alias()
      Use FmtDate In Select()
      Select FmtDate		&& If created in Init, it will have no records
      If RecCount() = 0			&& Need one record in DBF
         Append Blank
      Endif
      If Set("Date") = "DMY"
         Set Date To MDY
         this.Text = "dmy"
         this.Speedtip = "Toggle the date format to D/M/Y"
         Replace FmtDate->MDYDMY With "MDY"
      Else 
         Set Date To DMY
         this.Text = "mdy"
         this.Speedtip = "Toggle the date format to M/D/Y"
         Replace FmtDate->MDYDMY With "DMY"
      Endif
      Use in FmtDate
      If .Not. Empty (cAlias)		&& User did have a table open
         Select &cAlias
      Else				&& There were no tables open
         Select Select()
      Endif
      form.Refresh()

   Procedure Init
   *---------Code for creating FMTDATE.DBF
    if .not. file ( "FMTDATE.DBF" )
       create table "FMTDATE.DBF" ( ;
       FMTDATE.'CENTURY'     CHAR(3),;
       FMTDATE.'MDYDMY'      CHAR(3),;
       FMTDATE.'MARK'        CHAR(1))
    endif
   *---------End FMTDATE.DBF

   
ENDCLASS

*-- Button to toggle between a two digit and a four digit year format

CLASS CENTURYBUTTON(FormObj,Name) OF PUSHBUTTON(FormObj,Name) Custom
   this.Top = 16
   this.Text = "yy"
   this.SpeedTip = "Toggle the date format between MM/DD/YY and MM/DD/YYYY."
   this.OnClick = CLASS::CENTURYBUTTON_ONCLICK
   this.Left = 20
   this.Group = .T.
   this.Height = 1.53
   this.Width = 13
   this.UpBitmap = "RESOURCE #625"	&&#1130"
   this.OnOpen = CLASS::OnOpen
   
   Procedure OnOpen
      Class::Init()			&& Create FmtDate.dbf if it doesn't exist
      Private cAlias
      cAlias = Alias()			&& Assume user has a table open
      Use FmtDate In Select()
      Select FmtDate
      If RecCount() = 0			&& Need one record in DBF
         Append Blank
      Endif
      If Empty(Century)			&& Read from the MyApp.INI file
         If Set("Century") = "ON"
            this.Text = "yy"
            this.Speedtip = "Toggle the date format to MM/DD/YY"
         Else
            this.Text = "yyyy"
            this.Speedtip = "Toggle the date format to MM/DD/YYYY"
         Endif
      Else				&& Read from the FmtDate.dbf file
         If FmtDate->Century = "ON"
            Set Century ON
            this.text = "yy"
            this.Speedtip = "Toggle the date format to MM/DD/YY"
         Else
            Set Century OFF
            this.text = "yyyy"
            this.Speedtip = "Toggle the date format to MM/DD/YYYY"
         Endif
      Endif
      Use in FmtDate
      If .Not. Empty (cAlias)		&& User did have a table open
         Select &cAlias
      Else				&& There were no tables open
         Select Select()
      Endif

   Procedure CENTURYBUTTON_OnClick
      cAlias = Alias()
      Use FmtDate In Select()
      Select FmtDate
      If Set("Century") = "OFF"
         Set Century On
         this.Text = "yy"
         this.Speedtip = "Toggle the date format to MM/DD/YY"
         Replace FmtDate->Century With "ON"
      Else 
         Set Century Off
         this.Text = "yyyy"
         this.Speedtip = "Toggle the date format to MM/DD/YYYY"
         Replace FmtDate->Century With "OFF"
      Endif
      Use in FmtDate
      If .Not. Empty (cAlias)		&& User did have a table open
         Select &cAlias
      Else				&& There were no tables open
         Select Select()
      Endif
      form.Refresh()

   Procedure Init
   *---------Code for creating FMTDATE.DBF
    if .not. file ( "FMTDATE.DBF" )
       create table "FMTDATE.DBF" ( ;
       FMTDATE.'CENTURY'     CHAR(3),;
       FMTDATE.'MDYDMY'      CHAR(3),;
       FMTDATE.'MARK'        CHAR(1))
    endif
   *---------End FMTDATE.DBF

ENDCLASS


*-- Button to toggle between a "/" and a "." as the date separator.
*-- This should be the last property changed when format is modified.

CLASS MarkBUTTON(FormObj,Name) OF PUSHBUTTON(FormObj,Name) Custom
   this.Top = 16
   this.Text = "Mark /"
   this.SpeedTip = "Toggle the date format between M/D/Y and M.D.Y."
   this.OnClick = CLASS::MARKBUTTON_ONCLICK
   this.Left = 20
   this.Group = .T.
   this.Height = 1.53
   this.Width = 13
   this.UpBitmap = "RESOURCE #625"	&&#1130"
   this.OnOpen = CLASS::OnOpen
   
   Procedure OnOpen
      Class::Init()			&& Create FmtDate.dbf if it doesn't exist
      Private cAlias
      cAlias = Alias()			&& Assume user has a table open
      Use FmtDate In Select()
      Select FmtDate
      If Empty(Mark)			&& Read from the MyApp.INI file
         If Set("Mark") = "/"
            this.Text = "Mark ."
         Else
            this.Text = "Mark /"
         Endif
      Else				&& Read from the FmtDate.dbf file
         If FmtDate->Mark = "/"
            Set Mark To "/"
            this.text = "Mark ."
            this.Speedtip = "Toggle the date format to D.M.Y"
         Else
            Set Mark To "."
            this.text = "Mark /"
            this.Speedtip = "Toggle the date format to D/M/Y"
         Endif
      Endif
      Use in FmtDate
      If .Not. Empty (cAlias)		&& User did have a table open
         Select &cAlias
      Else				&& There were no tables open
         Select Select()
      Endif

   Procedure MARKBUTTON_OnClick
      cAlias = Alias()
      Use FmtDate In Select()
      Select FmtDate
      If Set("Mark") = "."
         Set Mark To "/"
         this.Text = "Mark ."
         this.Speedtip = "Toggle the date format to D.M.Y"
         Replace FmtDate->Mark With "/"
      Else 
         Set Mark To "."
         this.Text = "Mark /"
         this.Speedtip = "Toggle the date format to D/M/Y"
         Replace FmtDate->Mark With "."
      Endif
      Use in FmtDate
      If .Not. Empty (cAlias)		&& User did have a table open
         Select &cAlias
      Else				&& There were no tables open
         Select Select()
      Endif
      form.Refresh()

   Procedure Init
   *---------Code for creating FMTDATE.DBF
    if .not. file ( "FMTDATE.DBF" )
       create table "FMTDATE.DBF" ( ;
       FMTDATE.'CENTURY'     CHAR(3),;
       FMTDATE.'MDYDMY'      CHAR(3),;
       FMTDATE.'MARK'        CHAR(1))
    endif
   *---------End FMTDATE.DBF
   
ENDCLASS
